@@ -68,7 +68,7 @@ public class IOUtil {
     /** {@value} */
     public static final String JAR_SCHEME = "jar";
     /** A JAR subprotocol is separeted from the JAR entry w/ this separator {@value}. Even if no class is specified '!/' must follow!. */
-    public static final String JAR_SCHEME_SEPARATOR = "!";
+    public static final char JAR_SCHEME_SEPARATOR = '!';
 
     /** Std. temporary directory property key <code>java.io.tmpdir</code>. */
     private static final String java_io_tmpdir_propkey = "java.io.tmpdir";
@@ -313,7 +313,7 @@ public class IOUtil {
      * @throws URISyntaxException if the resulting string does not comply w/ an RFC 2396 URI
      */
     public static URI toURISimple(File file) throws URISyntaxException {
-        return new URI(FILE_SCHEME, null, encodeToURI(slashify(file.getAbsolutePath(), true, file.isDirectory())), null);
+        return new URI(FILE_SCHEME, null, slashify(file.getAbsolutePath(), true /* startWithSlash */, file.isDirectory() /* endWithSlash */), null);
     }
 
     /**
@@ -321,8 +321,8 @@ public class IOUtil {
      * @throws URISyntaxException if path is empty or has no parent directory available while resolving <code>../</code>
      * @throws URISyntaxException if the resulting string does not comply w/ an RFC 2396 URI
      */
-    public static URI toURISimple(String protocol, String file, boolean isDirectory) throws URISyntaxException {
-        return new URI(protocol, null, encodeToURI(slashify(file, true, isDirectory)), null);
+    public static URI toURISimple(String protocol, String path, boolean isDirectory) throws URISyntaxException {
+        return new URI(protocol, null, slashify(new File(path).getAbsolutePath(), true /* startWithSlash */, isDirectory /* endWithSlash */), null);
     }
 
     /**
@@ -412,7 +412,7 @@ public class IOUtil {
      * @throws URISyntaxException if path is empty or has no parent directory available while resolving <code>../</code>
      */
     public static String getBasename(String fname) throws URISyntaxException {
-        fname = slashify(fname, false, false);
+        fname = slashify(fname, false /* startWithSlash */, false /* endWithSlash */);
         int lios = fname.lastIndexOf('/');  // strip off dirname
         if(lios>=0) {
             fname = fname.substring(lios+1);
@@ -425,7 +425,7 @@ public class IOUtil {
      * @throws URISyntaxException if path is empty or has no parent directory available while resolving <code>../</code>
      */
     public static String getDirname(String fname) throws URISyntaxException {
-        fname = slashify(fname, false, false);
+        fname = slashify(fname, false /* startWithSlash */, false /* endWithSlash */);
         int lios = fname.lastIndexOf('/');  // strip off dirname
         if(lios>=0) {
             fname = fname.substring(0, lios+1);
@@ -451,7 +451,7 @@ public class IOUtil {
         }
         String uriS = uri.toString();
         if( DEBUG ) {
-            System.out.println("getURIDirname "+uri+", extForm: "+uriS);
+            System.err.println("getURIDirname "+uri+", extForm: "+uriS);
         }
         return new URI( getURIDirname(uriS) );
     }
@@ -487,23 +487,32 @@ public class IOUtil {
         uriS = uriS.substring(0, idx+1); // exclude jar name, include terminal '/' or ':'
 
         if( DEBUG ) {
-            System.out.println("getJarURIDirname res: "+uriS);
+            System.err.println("getJarURIDirname res: "+uriS);
         }
         return uriS;
     }
 
     /**
-     * Converts an {@link URI} to an {@link URL} while using a non encoded path
-     * for <i>file scheme</i>, i.e. <code>file:/</code>.
+     * Converts an {@link URI} to an {@link URL} while using a non encoded path.
+     * <p>
+     * A <i>file scheme</i> path, i.e. path following <code>file:</code>, is converted as follows:<br/>
+     * <code><br/>
+            File file = new File( {@link #decodeFromURI(String) decodeFromURI}( specificURI.getPath() ) );<br/>
+            String uriFilePath = {@link #encodeFilePathToURI(String) encodeFilePathToURI}( file.getPath() );<br/>
+     * </code><br/>
+     * above conversion results in a decoded <i>file path</i> appropriate to be used by subsequent file i/o operations (JarFile, zip, ..).
+     * </p>
+     * <p>
      * Otherwise the default {@link URL} translation {@link URI#toURL()} is being used.
+     * </p>
      * <p>
      * The following cases are considered:
      * <ul>
      *   <li><i>file schema</i> is converted via <code>new File(uri).getPath()</code>.</li>
      *   <li><i>jar scheme</i>
      *   <ul>
-     *     <li>sub-protocol is being converted as above, if <i>file scheme</i>.</li>
-     *     <li>JAR entry is not converted but preserved.</li>
+     *     <li>sub-protocol <i>file scheme</i> is being converted as above, other schema are preserved</li>
+     *     <li>JAR entry is preserved.</li>
      *   </ul></li>
      * </ul>
      * </p>
@@ -523,18 +532,23 @@ public class IOUtil {
         final URI specificURI = isJar ? JarUtil.getJarSubURI(uri) : uri;
         final boolean hasJarSubURI = specificURI != uri;
         if( DEBUG ) {
-            System.out.println("IOUtil.toURL.0: isJAR "+isJar+", hasSubURI "+hasJarSubURI+Platform.getNewline()+
-                               "\t, uri "+uri+Platform.getNewline()+
-                               "\t -> "+specificURI.toString());
+            System.err.println("IOUtil.toURL.0: isJAR "+isJar+", hasSubURI "+hasJarSubURI+PlatformPropsImpl.NEWLINE+
+                               "\t, uri "+uri+PlatformPropsImpl.NEWLINE+
+                               "\t str -> "+specificURI.toString()+PlatformPropsImpl.NEWLINE+
+                               "\t ascii -> "+specificURI.toASCIIString()+PlatformPropsImpl.NEWLINE+
+                               "\t ssp -> "+specificURI.getSchemeSpecificPart()+PlatformPropsImpl.NEWLINE+
+                               "\t frag -> "+specificURI.getFragment()+PlatformPropsImpl.NEWLINE+
+                               "\t path -> "+specificURI.getPath()+PlatformPropsImpl.NEWLINE+
+                               "\t path.decoded -> "+decodeFromURI( specificURI.getPath() ) );
         }
         int mode = 0;
         if( IOUtil.FILE_SCHEME.equals( specificURI.getScheme() ) ) {
             File f;
             try {
-                f = new File(specificURI);
+                f = new File( decodeFromURI( specificURI.getPath() ) ); // validates uri, uses decoded uri.getPath() and normalizes it
             } catch(Exception iae) {
                 if( DEBUG ) {
-                    System.out.println("Catched "+iae.getClass().getSimpleName()+": new File("+specificURI+") failed: "+iae.getMessage());
+                    System.err.println("Catched "+iae.getClass().getSimpleName()+": new File("+decodeFromURI( specificURI.getPath() )+") failed: "+iae.getMessage());
                     iae.printStackTrace();
                 }
                 f = null;
@@ -558,18 +572,18 @@ public class IOUtil {
                         try {
                             final URI fUri = f.toURI();
                             final URL fUrl = fUri.toURL();
-                            System.out.println("IOUtil.toURL.1b: fUri "+fUri+Platform.getNewline()+
+                            System.err.println("IOUtil.toURL.1b: fUri "+fUri+PlatformPropsImpl.NEWLINE+
                                                "\t, fUrl "+fUrl);
                         } catch (Exception ee) {
-                            System.out.println("Catched "+ee.getClass().getSimpleName()+": f.toURI().toURL() failed: "+ee.getMessage());
+                            System.err.println("Catched "+ee.getClass().getSimpleName()+": f.toURI().toURL() failed: "+ee.getMessage());
                             ee.printStackTrace();
                         }
                     }
                     if( !hasJarSubURI ) {
                         urlS = IOUtil.FILE_SCHEME+IOUtil.SCHEME_SEPARATOR+fPathUriS;
                         if( DEBUG ) {
-                            System.out.println("IOUtil.toURL.1: fPath "+fPath+Platform.getNewline()+
-                                               "\t -> "+fPathUriS+Platform.getNewline()+
+                            System.err.println("IOUtil.toURL.1: fPath "+fPath+PlatformPropsImpl.NEWLINE+
+                                               "\t -> "+fPathUriS+PlatformPropsImpl.NEWLINE+
                                                "\t -> "+urlS);
                         }
                         url = new URL(urlS);
@@ -579,10 +593,10 @@ public class IOUtil {
                         final String post = isJar ? IOUtil.JAR_SCHEME_SEPARATOR + jarEntry : "";
                         urlS = uriSchema+IOUtil.SCHEME_SEPARATOR+IOUtil.FILE_SCHEME+IOUtil.SCHEME_SEPARATOR+fPathUriS+post;
                         if( DEBUG ) {
-                            System.out.println("IOUtil.toURL.2: fPath "+fPath+Platform.getNewline()+
-                                               "\t -> "+fPathUriS+Platform.getNewline()+
-                                               "\t, jarEntry "+jarEntry+Platform.getNewline()+
-                                               "\t, post "+post+Platform.getNewline()+
+                            System.err.println("IOUtil.toURL.2: fPath "+fPath+PlatformPropsImpl.NEWLINE+
+                                               "\t -> "+fPathUriS+PlatformPropsImpl.NEWLINE+
+                                               "\t, jarEntry "+jarEntry+PlatformPropsImpl.NEWLINE+
+                                               "\t, post "+post+PlatformPropsImpl.NEWLINE+
                                                "\t -> "+urlS);
                         }
                         url = new URL(urlS);
@@ -590,7 +604,7 @@ public class IOUtil {
                     }
                 } catch (Exception mue) {
                     if( DEBUG ) {
-                        System.out.println("Catched "+mue.getClass().getSimpleName()+": new URL("+urlS+") failed: "+mue.getMessage());
+                        System.err.println("Catched "+mue.getClass().getSimpleName()+": new URL("+urlS+") failed: "+mue.getMessage());
                         mue.printStackTrace();
                     }
                 }
@@ -602,13 +616,13 @@ public class IOUtil {
                 mode = 3;
             } catch (Exception e) {
                 if( DEBUG ) {
-                    System.out.println("Catched "+e.getClass().getSimpleName()+": "+uri+".toURL() failed: "+e.getMessage());
+                    System.err.println("Catched "+e.getClass().getSimpleName()+": "+uri+".toURL() failed: "+e.getMessage());
                     e.printStackTrace();
                 }
             }
         }
         if( DEBUG ) {
-            System.err.println("IOUtil.toURL.X: mode "+mode+", "+uri+Platform.getNewline()+
+            System.err.println("IOUtil.toURL.X: mode "+mode+", "+uri+PlatformPropsImpl.NEWLINE+
                                "\t -> "+url);
         }
         return url;
@@ -722,7 +736,7 @@ public class IOUtil {
         if (baseLocation != null) {
             final File file = new File(baseLocation, relativeFile);
             // Handle things on Windows
-            return slashify(file.getPath(), false, false);
+            return slashify(file.getPath(), false /* startWithSlash */, false /* endWithSlash */);
         }
         return null;
     }
@@ -790,7 +804,7 @@ public class IOUtil {
      * @throws URISyntaxException if path is empty or has no parent directory available while resolving <code>../</code>
      */
     public static URI getRelativeOf(URI baseURI, String relativePath) throws URISyntaxException {
-        return compose(baseURI.getScheme(), baseURI.getSchemeSpecificPart(), encodeToURI(relativePath), baseURI.getFragment());
+        return compose(baseURI.getScheme(), baseURI.getSchemeSpecificPart(), relativePath, baseURI.getFragment());
     }
 
     /**
@@ -857,17 +871,21 @@ public class IOUtil {
 
     /**
      * Reverses escaping of characters as performed via {@link #encodeToURI(String)}.
+     * <ul>
+     *   <li>%20 -> SPACE</li>
+     * </ul>
      */
     public static String decodeFromURI(String s) {
         return patternSpaceEnc.matcher(s).replaceAll(" ");
     }
 
     private static final Pattern patternSingleBS = Pattern.compile("\\\\{1,}");
+    private static final Pattern patternSingleFS = Pattern.compile("/{1,}");
 
     /**
-     * Escapes file path characters not complying w/ RFC 2396 and the {@link URI#URI(String)} ctor.
+     * Encodes file path characters not complying w/ RFC 2396 and the {@link URI#URI(String)} ctor.
      * <p>
-     * Processes input filePath if {@link File#separatorChar} <code> != '/'</code>
+     * Implementation processes the <code>filePath</code> if {@link File#separatorChar} <code> != '/'</code>
      * as follows:
      * <ul>
      *   <li>backslash -> slash</li>
@@ -875,12 +893,17 @@ public class IOUtil {
      * </ul>
      * </p>
      * <p>
+     * Note that this method does not perform <i>space</i> encoding,
+     * which can be utilized via {@link #encodeToURI(String)}.
+     * </p>
+     * <p>
      * Even though Oracle's JarURLStreamHandler can handle backslashes and
      * erroneous URIs w/ e.g. Windows file 'syntax', other may not (Netbeans).<br>
      * See Bug 857 - http://jogamp.org/bugzilla/show_bug.cgi?id=857
      * </p>
+     * @see #encodeToURI(String)
      */
-    public static String encodeFilePathToURI(String filePath) {
+    public static String encodeFilePathToURI(final String filePath) {
         if( !File.separator.equals("/") ) {
             final String r = patternSingleBS.matcher(filePath).replaceAll("/");
             if( !r.startsWith("/") ) {
@@ -892,6 +915,49 @@ public class IOUtil {
         return filePath;
     }
 
+    /**
+     * Decodes uri-file path characters complying w/ RFC 2396 to native file-path.
+     * <p>
+     * Implementation decodes the space-encoding <code>path={@link #decodeFromURI(String) decodeFromURI}(uriPath)</code>.
+     * </p>
+     * <p>
+     * Then it processes the <code>path</code> if {@link File#separatorChar} <code> != '/'</code>
+     * as follows:
+     * <ul>
+     *   <li>drop a starting slash</li>
+     *   <li>slash -> backslash</li>
+     * </ul>
+     * </p>
+     * @see #decodeFromURI(String)
+     */
+    public static String decodeURIToFilePath(final String uriPath) {
+        final String path = IOUtil.decodeFromURI(uriPath);
+        if( !File.separator.equals("/") ) {
+            final String r = patternSingleFS.matcher(path).replaceAll("\\\\");
+            if( r.startsWith("\\") ) {
+                return r.substring(1);
+            } else {
+                return r;
+            }
+        }
+        return path;
+    }
+
+    /**
+     * If <code>uri</code> is a <i>file scheme</i>,
+     * implementation returns the decoded {@link URI#getPath()} via {@link #decodeURIToFilePath(String)},
+     * otherwise it returns the {@link URI#toASCIIString()} encoded URI.
+     *
+     * @see #decodeFromURI(String)
+     * @see #decodeURIToFilePath(String)
+     */
+    public static String decodeURIIfFilePath(final URI uri) {
+        if( IOUtil.FILE_SCHEME.equals( uri.getScheme() ) ) {
+            return decodeURIToFilePath( uri.getPath() );
+        }
+        return uri.toASCIIString();
+    }
+
     /**
      * Returns the connected URLConnection, or null if not url is not available
      */
@@ -1202,7 +1268,7 @@ public class IOUtil {
 
                     // 1) java.io.tmpdir/jogamp
                     if( null == tempRootExec && isStringSet(java_io_tmpdir) ) {
-                        if( Platform.OSType.MACOS == Platform.getOSType() ) {
+                        if( Platform.OSType.MACOS == PlatformPropsImpl.OS_TYPE ) {
                             // Bug 865: Safari >= 6.1 [OSX] May employ xattr on 'com.apple.quarantine' on 'PluginProcess.app'
                             // We attempt to fix this issue _after_ gluegen native lib is loaded, see JarUtil.fixNativeLibAttribs(File).
                             tempRootExec = getSubTempDir(new File(java_io_tmpdir), tmpSubDir, false /* executable */, "tempX1");
